#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>

#include "harpa.h"

void init_PWM(void) {
	TCCR1A |= (1 << COM1A0) | (1 << WGM10);
	TCCR1B |= _BV(WGM13) | _BV(CS12);
}

void set_PWM(void) {
	TCCR1A |= (1 << COM1A0);	// enable output
}

void unset_PWM(void) {
	OCR1A = 255;
	TCCR1A &= ~(1 << COM1A0);	// disable output
}

void init_Timer0(void) {		// timer 0 for 16 ms interrupt
	TCCR0A |= (1 << COM0A1);	// clear OC0A on compare match
	TCCR0A |= (1 << WGM01);	// CTC
	TCCR0B |= (1 << CS02) | (1 << CS00);	// prescaler clk/1024
	TIMSK0 |= (1 << OCIE0A);
	
	OCR0A = 255;
}

void init_ADC(void) {
	ADMUX &= 0xE0;
	ADMUX |= (1<<REFS0);		// voltage reference from Avcc (5v)
	
	ADCSRA = 0;
	ADCSRA |= ((1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0));		// 16Mhz/128 = 125Khz the ADC reference clock
	ADCSRA |= (1<<ADEN);		// turn on ADC
	ADCSRA |= (1<<ADSC);		// do an initial conversion because this one is the slowest and to ensure that everything is up and running
}
 
int get_ADC(uint8_t channel) {	
	ADMUX &= 0xE0;				// keep first 3 bits unchanged
	ADMUX |= channel;			// mark channel for read
	
	ADCSRA |= (1 << ADSC);		// starts a new conversion
	while(!bit_is_set(ADCSRA,ADIF));	// wait until the conversion is done: until ADIF from ADCSRA is set to 1
	return ADC;					// returns the ADC value of the chosen channel
}

void set_note(int note) {
	OCR1A = notes[note];		// set sound to specified note
}

ISR(TIMER0_COMPA_vect) {
	int i;
	for (i = 0; i < 8; i++) {
		int val = get_ADC(i);	// get value from current pin
		if (val > 300) {		// if laser interrupted
			PORTD |= (1 << PD7);
			set_note(i);		// play sound
			set_PWM();
			break;
		} else {
			PORTD &= ~(1 << PD7);
			unset_PWM();
		}
	}
}


int main(void)
{
	DDRD |= (1 << PD5) | (1 << PD7); // difuzor + user led
	PORTD |= (1 << PD7);	

	init_Timer0();
	init_ADC();
	init_PWM();
	
	sei();		// enable global interrupts

    while (1);	// wait for interrupts
	
	return 0;
}